// file: analog.ino
//
// Cte analogovy vstup a podle nastavenych limitu aktivuje LED diody.
// Jsou zde celkem dva kanaly. Stejne chovani i pro druhy kanal.
// Kanal 0 (ch0)->(A6,PD2,PD3,PD4)
// Kanal 1 (ch1)->(A7,PD5,PD6,PD7)
//
// y(V)=(5V/255).x(n)
// x=12->  y=0.235V
// x=39->  y=0.765V
// x=123-> y=2.41V
// x=131-> y=2.57V
//
// otestovano: n/*y @ arduino pro mini 328p @ 8MHz @ 5V
// IDE 1.0.2
//
// A6; analog 6; potenciometr
// D2; DO; PD2; limit 0; led '0' - sviti {lim0l..lim0h}
// D3; DO; PD3; limit 1; led '0' - sviti {lim1l..lim1h}
// D4; DO; PD4; out of limit; led '0' - sviti {default}
//
// A7; analog 7; potenciometr
// D5; DO; PD5; limit 0; led '0' - sviti {lim0l..lim0h}
// D6; DO; PD6; limit 1; led '0' - sviti {lim1l..lim1h}
// D7; DO; PD7; out of limit; led '0' - sviti {default}
// - - - - - - - - - - - - - - -
#define DEBUG (0)

#if (DEBUG==1)
#include <Dbg.h>
#endif

// - - - - - - - - - - - - - - -
const int sensorA6 = A6;
const int sensorA7 = A7;

// LED shines if '0' is written to PORTD

#define LED_xxx000xx {temp=PORTD;\
                      temp|=0x1C;\
					  PORTD=temp;}

#define LED_xxx001xx {temp=PORTD;\
                      temp|=0x1C;\
                      temp&=~(0x04);\
					  PORTD=temp;}

#define LED_xxx010xx {temp=PORTD;\
                      temp|=0x1C;\
                      temp&=~(0x08);\
					  PORTD=temp;}

#define LED_xxx100xx {temp=PORTD;\
                      temp|=0x1C;\
                      temp&=~(0x10);\
					  PORTD=temp;}

#define LED_000xxxxx {temp=PORTD;\
                      temp|=0xE0;\
					  PORTD=temp;}

#define LED_001xxxxx {temp=PORTD;\
                      temp|=0xE0;\
                      temp&=~(0x20);\
					  PORTD=temp;}

#define LED_010xxxxx {temp=PORTD;\
                      temp|=0xE0;\
                      temp&=~(0x40);\
					  PORTD=temp;}

#define LED_100xxxxx {temp=PORTD;\
                      temp|=0xE0;\
                      temp&=~(0x80);\
					  PORTD=temp;}

#define LED_SETUP {PORTD=0b11111100; \
                    DDRD=0b11111100;}
// - - - - - - - - - - - - - - -
void get_ad( void);
void set_output_ch0( void);
void set_output_ch1( void);

// - - - - - - - - - - - - - - -
typedef struct
{
  const uint8_t lim0l;
  const uint8_t lim0h;
  const uint8_t lim1l;
  const uint8_t lim1h;
} t_limits;

t_limits lim =
{
  (12) /*lim0l*/ ,\
  (39) /*lim0h*/ ,\
  (123) /*lim1l*/ ,\
  (131) /*lim1h*/ \
};



enum enu_status \
{
  _outOfLimit=0x00, \
  _lim0=0x01, \
  _lim1=0x02\
};


// - - - - - - - - - - - - - - -
uint8_t temp;
uint8_t ad6;
uint8_t ad7;
enu_status sb1;
enu_status sb2;

// - - - - - - - - - - - - - - -
void setup()
{
  LED_SETUP;

#if (DEBUG==1)
  Debug.begin(1,9600);
#endif
}


// - - - - - - - - - - - - - - -
void loop()
{
  get_ad();
  set_output_ch0();
  set_output_ch1();

#if (DEBUG==1)
  dbg("%d\t%d", ad7, (uint8_t) sb2);
#endif
}

// - - - - - - - - - - - - - - -
void set_output_ch0( void)
{
  temp = ad6;

  if ((temp>lim.lim0l)&&(temp<lim.lim0h))
  {
    sb1 = _lim0; //0x01
  }
  else if ((temp>lim.lim1l)&&(temp<lim.lim1h))
  {
    sb1 = _lim1; //0x02
  }
  else
  {
    sb1 = _outOfLimit; //0x00
  }

  switch (sb1)
  {
  case _outOfLimit :
  {
    LED_xxx100xx
    break;
  }
  case _lim0 :
  {
    LED_xxx001xx;
    break;
  }
  case _lim1 :
  {
    LED_xxx010xx;
    break;
  }
  default :
  {
    LED_xxx000xx
    break;
  }
  };
}
// - - - - - - - - - - - - - - -
void set_output_ch1( void)
{
  temp = ad7;

  if ((temp>lim.lim0l)&&(temp<lim.lim0h))
  {
    sb2 = _lim0; //0x01
  }
  else if ((temp>lim.lim1l)&&(temp<lim.lim1h))
  {
    sb2 = _lim1; //0x02
  }
  else
  {
    sb2 = _outOfLimit; //0x00
  }

  switch (sb2)
  {
  case _outOfLimit :
  {
    LED_100xxxxx;
    break;
  }
  case _lim0 :
  {
    LED_001xxxxx;
    break;
  }
  case _lim1 :
  {
    LED_010xxxxx;
    break;
  }
  default :
  {
    LED_000xxxxx;
    break;
  }
  };
}
// - - - - - - - - - - - - - - -
void get_ad( void)
{
  ad6=(analogRead(sensorA6)>>2); // 8bit only
  ad7=(analogRead(sensorA7)>>2); // 8bit only
}
// EOF

